/*
 * ContactsContactReader.java
 *
 * Copyright (C) 2011 Tim Marston <edam@waxworlds.org>
 *
 * This file is part of the Export Contacts program (hereafter referred
 * to as "this program"). For more information, see
 * http://www.waxworlds.org/edam/software/android/export-contacts
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.waxworlds.edam.exportcontacts;

import android.app.Activity;
import android.database.Cursor;
import android.provider.Contacts;

public class ContactsContactReader implements ContactReader
{
	Activity _activity = null;
	Exporter _exporter = null;
	Cursor _cur = null;

	public ContactsContactReader( Activity activity, Exporter exporter )
	{
		_activity = activity;
		_exporter = exporter;
	}

	@Override
	public int getNumContacts()
	{
		Cursor cursor = _activity.managedQuery(
			Contacts.People.CONTENT_URI,
			new String[] {
				Contacts.People._ID,
			}, null, null, null );
		return cursor.getCount();
	}

	@Override
	public boolean getNextContact( Exporter.ContactData contact )
	{
		// set up cursor
		if( _cur == null )
		{
			// get all contacts
			_cur = _activity.managedQuery( Contacts.People.CONTENT_URI,
				new String[] {
					Contacts.People._ID,
					Contacts.People.NAME,
				}, null, null, null );
		}

		// if there are no more contacts, abort
		if( !_cur.moveToNext() ) {
			_cur = null;
			return false;
		}

		// get this contact's id
		Long id = _cur.getLong( _cur.getColumnIndex( Contacts.People._ID ) );

		// create contact
		contact.setName(
			_cur.getString( _cur.getColumnIndex( Contacts.People.NAME ) ) );

		// get the organisations
		Cursor cur = _activity.managedQuery( Contacts.Organizations.CONTENT_URI,
			new String[] {
				Contacts.Organizations.COMPANY,
				Contacts.Organizations.TITLE,
			}, Contacts.Organizations.PERSON_ID + " = ?",
			new String[] { id.toString() },
			Contacts.Organizations.ISPRIMARY + " DESC, " +
				Contacts.Organizations.PERSON_ID + " ASC" );
		while( cur.moveToNext() )
			contact.addOrganisation( contact.new OrganisationDetail(
				cur.getString( cur.getColumnIndex(
					Contacts.Organizations.COMPANY ) ),
				cur.getString( cur.getColumnIndex(
					Contacts.Organizations.TITLE ) ) ) );
		cur.close();

		// get the phone numbers
		cur = _activity.managedQuery( Contacts.Phones.CONTENT_URI,
			new String[] {
				Contacts.Phones.NUMBER,
				Contacts.Phones.TYPE,
			}, Contacts.Phones.PERSON_ID + " = ?",
			new String[] { id.toString() },
			Contacts.Phones.ISPRIMARY + " DESC," +
				Contacts.Phones.PERSON_ID + " ASC" );
		while( cur.moveToNext() )
			contact.addNumber( contact.new NumberDetail(
				cur.getInt( cur.getColumnIndex(
					Contacts.Phones.TYPE ) ),
				cur.getString( cur.getColumnIndex(
					Contacts.Phones.NUMBER ) ) ) );
		cur.close();

		// get the email and postal addresses
		cur = _activity.managedQuery( Contacts.ContactMethods.CONTENT_URI,
			new String[] {
				Contacts.ContactMethods.KIND,
				Contacts.ContactMethods.TYPE,
				Contacts.ContactMethods.DATA,
			},
			Contacts.ContactMethods.PERSON_ID + " = ? AND " +
				Contacts.ContactMethods.KIND + " IN( ?, ? )",
			new String[] {
				id.toString(),
				"" + Contacts.KIND_EMAIL,
				"" + Contacts.KIND_POSTAL,
			},
			Contacts.ContactMethods.ISPRIMARY + " DESC," +
				Contacts.ContactMethods.PERSON_ID + " ASC" );
		while( cur.moveToNext() ) {
			int kind = cur.getInt( cur.getColumnIndex(
				Contacts.ContactMethods.KIND ) );
			if( kind == Contacts.KIND_EMAIL )
				contact.addEmail( contact.new EmailDetail(
					cur.getInt( cur.getColumnIndex(
						Contacts.ContactMethods.TYPE ) ),
					cur.getString( cur.getColumnIndex(
						Contacts.ContactMethods.DATA ) ) ) );
			else
				contact.addAddress( contact.new AddressDetail(
					cur.getInt( cur.getColumnIndex(
						Contacts.ContactMethods.TYPE ) ),
					cur.getString( cur.getColumnIndex(
						Contacts.ContactMethods.DATA ) ) ) );
		}
		cur.close();

		return true;
	}
}
